==> AV3D-src-1.1.zip: src/java/org/j3d/aviatrix3d/Scene.java <==
/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
// None

/**
 * Representation of the top level structure of a piece of scene graph that
 * can form a coherent rendering.
 * <p>
 *
 * A scene encapsulates a viewpoint that is used to view the scene from,
 * the geometry structure to render, data defining the view environment (eg
 * projection type) and global data, such as the background, global fog etc.
 * <p>
 *
 * A scene may also provide an instance {@link RenderEffectsProcessor} that
 * allows the user to provide pre or post processing effects on a per-scene
 * basis. Note that if you are running layers, this will be pre and post
 * processing per layer, not per final rendering. The processor allows a
 * limited form of immediate-mode rendering.
 *
 * @author Justin Couch
 * @version $Revision: 1.7 $
 */
public class Scene extends SceneGraphObject
{
    /** User-defined data storage */
    private Object userData;

    /** Counter to know whether this scene is live or not */
    private int liveCount;

    /** The Scene Graph renderableObjects */
    private Group renderableObjects;

    /** The current viewpoint instance */
    private Viewpoint currentViewpoint;

    /** The current fog instance */
    private Fog currentFog;

    /** The current background instance */
    private Background currentBackground;

    /** Current view environment */
    private ViewEnvironment viewEnvironment;

    /** Current effects processor */
    private RenderEffectsProcessor processor;

    /**
     * Create a default instance of this scene with no content provided.
     */
    public Scene()
    {
        liveCount = 0;
    }

    //----------------------------------------------------------
    // Methods defined by ScenegraphObject
    //----------------------------------------------------------

    /**
     * Notification that this object is live now. Overridden to make sure that
     * the live state of the nodes represents the same state as the parent
     * scene graph.
     *
     * @param state true if this should be marked as live now
     */
    protected void setLive(boolean state)
    {
        if(state)
            liveCount++;
        else if(liveCount > 0)
            liveCount--;

        if((liveCount == 0) || !alive)
        {
            super.setLive(state);

            if(renderableObjects != null)
                renderableObjects.setLive(state);
        }
    }

    /**
     * Set the scenegraph update handler for this node.  It will notify
     * all its children of the value. A null value will clear the current
     * handler.
     *
     * @param handler The instance to use as a handler
     */
    protected void setUpdateHandler(NodeUpdateHandler handler)
    {
        super.setUpdateHandler(handler);

        if(renderableObjects != null)
            renderableObjects.setUpdateHandler(handler);
    }

    //----------------------------------------------------------
    // Local Methods
    //----------------------------------------------------------

    /**
     * Register the scene processor to be used for this scene for pre and
     * post rendering effects. Setting a null value will clear the current
     * instance.
     *
     * @param prc The instance to use or null
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setRenderEffectsProcessor(RenderEffectsProcessor prc)
        throws InvalidWriteTimingException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        processor = prc;
    }

    /**
     * Get the currently set scene processor instance. If none is set,
     * returns null.
     *
     * @return The current processor or null
     */
    public RenderEffectsProcessor getRenderEffectsProcessor()
    {
        return processor;
    }

    /**
     * Assign some arbitrary piece of user data to this scene. This data
     * is also sent back to the user through scene processor callbacks when
     * they are made. Setting a null value will clear the current instance.
     *
     * @param data some arbitrary data the user wants to provide
     */
    public void setUserData(Object data)
    {
        userData = data;
    }

    /**
     * Get the currently set instance of user data. If none provided, return
     * null.
     *
     * @return Some arbitrary piece of data or null
     */
    public Object getUserData()
    {
        return userData;
    }

    /**
     * Set the collection of geometry that should be rendered to this
     * texture.
     *
     * A null value will clear the current geometry and result in only
     * rendering the background, if set. if not set, then whatever the default
     * colour is, is used (typically black).
     *
     * @param geom The new geometry to use or null
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setRenderedGeometry(Group geom)
        throws InvalidWriteTimingException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        if(renderableObjects != null)
        {
            renderableObjects.setUpdateHandler(null);
            renderableObjects.setLive(false);
        }

        renderableObjects = geom;

        if(renderableObjects != null)
        {
            renderableObjects.setUpdateHandler(updateHandler);
            renderableObjects.setLive(alive);
        }
    }

    /**
     * Get the root of the currently rendered scene. If none is set, this will
     * return null.
     *
     * @return The current scene root or null.
     */
    public Group getRenderedGeometry()
    {
        return renderableObjects;
    }

    /**
     * Set the viewpoint path that should be applied to the current surface.
     * The output drawn will be a combination of this information and that
     * of the view environment.
     *
     * @param vp The instance of the active viewpoint to use
     * @throws IllegalArgumentException The path contains a SharedGroup or
     *    the node is not live
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setActiveView(Viewpoint vp)
        throws InvalidWriteTimingException, IllegalArgumentException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        currentViewpoint = vp;
    }

    /**
     * Get the currently set active view. If none is set, return null.
     *
     * @return The current view instance or null
     */
    public Viewpoint getActiveView()
    {
        return currentViewpoint;
    }

    /**
     * Set the fog that should be applied to the current surface.
     * The output drawn will be a combination of this information and that
     * of the view environment.
     *
     * @param fog The instance of the active fog node
     * @throws IllegalArgumentException The path contains a SharedGroup or
     *    the node is not live
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setActiveFog(Fog fog)
        throws InvalidWriteTimingException, IllegalArgumentException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        currentFog = fog;
    }

    /**
     * Get the currently set active view. If none is set, return null.
     *
     * @return The current view instance or null
     */
    public Fog getActiveFog()
    {
        return currentFog;
    }

    /**
     * Set the background path that should be applied to the current surface.
     * The output drawn will be a combination of this information and that
     * of the view environment.
     *
     * @param bg The instance of the active background
     * @throws IllegalArgumentException The path contains a SharedGroup or
     *    the node is not live
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setActiveBackground(Background bg)
        throws InvalidWriteTimingException, IllegalArgumentException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        currentBackground = bg;
    }

    /**
     * Get the currently set active view. If none is set, return null.
     *
     * @return The current view instance or null
     */
    public Background getActiveBackground()
    {
        return currentBackground;
    }

    /**
     * Set the view environment that is used to render this surface.
     *
     * @param env The environment instance to use for the render setup
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setViewEnvironment(ViewEnvironment env)
        throws InvalidWriteTimingException, IllegalArgumentException
    {
        if(updateHandler != null && !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        viewEnvironment = env;
    }

    /**
     * Get the currently set active view. If none is set, return null.
     *
     * @return The current view instance or null
     */
    public ViewEnvironment getViewEnvironment()
    {
        return viewEnvironment;
    }
}

==> AV3D-src-2.2.zip: src/java/org/j3d/aviatrix3d/Scene.java <==
/*****************************************************************************
 *                   Yumetech, Inc Copyright (c) 2004 - 2006
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
import org.j3d.aviatrix3d.rendering.RenderEffectsProcessor;

/**
 * Representation of the top level structure of a piece of scene graph that
 * can form a coherent rendering.
 * <p>
 *
 * A scene encapsulates a viewpoint that is used to view the scene from,
 * the geometry structure to render, data defining the view environment (eg
 * projection type) and global data, such as the background, global fog etc.
 * <p>
 *
 * A scene may also provide an instance {@link RenderEffectsProcessor} that
 * allows the user to provide pre or post processing effects on a per-scene
 * basis. Note that if you are running layers, this will be pre and post
 * processing per layer, not per final rendering. The processor allows a
 * limited form of immediate-mode rendering.
 *
 * @author Justin Couch
 * @version $Revision: 1.14 $
 */
public abstract class Scene extends SceneGraphObject
{
    /** Current view environment */
    protected ViewEnvironment viewEnvironment;

    /** Current effects processor */
    protected RenderEffectsProcessor processor;

    /**
     * Create a default instance of this scene with no content provided.
     */
    public Scene()
    {
        viewEnvironment = new ViewEnvironment();
    }

    //----------------------------------------------------------
    // Methods defined by ScenegraphObject
    //----------------------------------------------------------

    /**
     * Notification that this object is live now. Overridden to make sure that
     * the live state of the nodes represents the same state as the parent
     * scene graph.
     *
     * @param state true if this should be marked as live now
     */
    protected void setLive(boolean state)
    {
        // Ignore stuff that doesn't change the state
        if(state == alive)
            return;

        super.setLive(state);

        viewEnvironment.setLive(state);
    }

    /**
     * Set the scenegraph update handler for this node.  It will notify
     * all its children of the value. A null value will clear the current
     * handler.
     *
     * @param handler The instance to use as a handler
     */
    protected void setUpdateHandler(NodeUpdateHandler handler)
    {
        super.setUpdateHandler(handler);

        viewEnvironment.setUpdateHandler(handler);
    }

    //----------------------------------------------------------
    // Local Methods
    //----------------------------------------------------------

    /**
     * Register the scene processor to be used for this scene for pre and
     * post rendering effects. Setting a null value will clear the current
     * instance.
     *
     * @param prc The instance to use or null
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the ApplicationUpdateObserver callback method
     */
    public void setRenderEffectsProcessor(RenderEffectsProcessor prc)
        throws InvalidWriteTimingException
    {
        if(isLive() && updateHandler != null &&
           !updateHandler.isPickingPermitted())
            throw new InvalidWriteTimingException(getAppUpdateWriteTimingMessage());

        processor = prc;
    }

    /**
     * Get the currently set scene processor instance. If none is set,
     * returns null.
     *
     * @return The current processor or null
     */
    public RenderEffectsProcessor getRenderEffectsProcessor()
    {
        return processor;
    }

    /**
     * Get the currently set active view. If none is set, return null.
     *
     * @return The current view instance or null
     */
    public ViewEnvironment getViewEnvironment()
    {
        return viewEnvironment;
    }

    /**
     * Internal check to see if this scene has an update handler registered.
     * If it does, that is a sign that it currently has a parent registered for
     * it.
     */
    boolean hasParent()
    {
        return updateHandler != null;
    }

    /**
     * Set the viewport dimensions from the parent viewport. These dimensions
     * are pushed down through the scene to the viewport.
     *
     * @param x The lower left x coordinate for the view
     * @param y The lower left y coordinate for the view
     * @param width The width of the viewport in pixels
     * @param height The height of the viewport in pixels
     */
    void setViewportDimensions(int x, int y, int width, int height)
    {
        viewEnvironment.setViewportDimensions(x, y, width, height);
    }
}
